class MitoSiteApp {
  constructor(containerSelector = ".scrollWebcam") {
    this.containers = document.querySelectorAll(containerSelector)
    this.instances = new Map()
    this.model = null
    this.isDetecting = false
  }

  async init() {
    // Load COCO-SSD model using the official library
    try {
      // Load the model directly from the CDN
      this.model = await cocoSsd.load()
      console.log("TensorFlow model loaded successfully")
    } catch (err) {
      console.error("Error loading TensorFlow model:", err)
    }

    this.containers.forEach((container) => this.setupContainer(container))
  }

  async setupContainer(container) {
    const instanceData = {
      video: container.querySelector("video"),
      select: document.querySelector(".scrollWebcamSelect"),
      currentStream: null,
      boxes: [],
      isDrawing: false,
      startX: 0,
      startY: 0,
    }

    this.instances.set(container, instanceData)

    await this.setupWebcam(container)
    this.setupCanvas(container)
    this.setupHUD(container)

    // Start detection loop once everything is set up
    this.startDetection(container)
  }

  async startDetection(container) {
    const data = this.instances.get(container)
    const { video } = data

    if (!this.model) {
      console.error("TensorFlow model not loaded")
      return
    }

    // Wait for video to be ready
    if (video.readyState === 0) {
      video.addEventListener("loadeddata", () => this.startDetection(container))
      return
    }

    const detectFrame = async () => {
      if (!this.isDetecting) return

      // Check if video is ready and has valid dimensions
      if (
        video.readyState !== 4 ||
        video.videoWidth === 0 ||
        video.videoHeight === 0
      ) {
        requestAnimationFrame(() => detectFrame())
        return
      }

      try {
        // Make sure video dimensions are set
        if (video.width !== video.videoWidth) {
          video.width = video.videoWidth
        }
        if (video.height !== video.videoHeight) {
          video.height = video.videoHeight
        }

        // Perform detection only if video is playing and visible
        if (!video.paused && !video.ended && video.style.display !== "none") {
          const predictions = await this.model.detect(video)

          // Clear previous boxes
          this.clearCanvas(container)

          // Draw AI predictions in a different color without adding to boxes
          predictions.forEach((prediction) => {
            if (prediction.score > 0.5) {
              // Confidence threshold
              const [x, y, width, height] = prediction.bbox

              // Draw AI predictions in blue
              const { ctx } = data
              ctx.strokeStyle = "#0000ff"
              ctx.lineWidth = 2
              ctx.strokeRect(x, y, width, height)

              if (prediction.score !== null) {
                ctx.fillStyle = "#0000ff"
                ctx.font = "12px Arial"
                ctx.fillText(`${Math.round(prediction.score * 100)}%`, x, y - 5)
              }
            }
          })

          // Redraw manual boxes in green
          data.boxes.forEach((box) => {
            this.drawBox(container, box.x, box.y, box.width, box.height)
          })
        }
      } catch (err) {
        console.error("Detection error:", err)
      }

      // Request next frame
      requestAnimationFrame(() => detectFrame())
    }

    // Start detection loop
    this.isDetecting = true
    detectFrame()
  }

  drawBox(container, x, y, width, height, score = null) {
    const data = this.instances.get(container)
    const { ctx } = data

    ctx.strokeStyle = "#00ff00"
    ctx.lineWidth = 2
    ctx.strokeRect(x, y, width, height)

    if (score !== null) {
      ctx.fillStyle = "#00ff00"
      ctx.font = "12px Arial"
      ctx.fillText(`${Math.round(score * 100)}%`, x, y - 5)
    }
  }

  updateCellCount(container) {
    const data = this.instances.get(container)
    const hud = container.querySelector(".scrollWebcamHud")
    const cellCounter = hud.querySelector(
      '.scrollWebcamHudElement[data-type="cellCount"]',
    )
    if (cellCounter) {
      cellCounter.textContent = `Cells Detected: ${data.boxes.length}`
    }
  }

  async setupWebcam(container) {
    const data = this.instances.get(container)
    const { video, select } = data

    try {
      const devices = await navigator.mediaDevices.enumerateDevices()
      const videoDevices = devices.filter(
        (device) => device.kind === "videoinput",
      )

      // Add demo video option first
      const demoOption = document.createElement("option")
      demoOption.value = "demo"
      demoOption.text = "Demo Video"
      select.appendChild(demoOption)

      // Add webcam options
      videoDevices.forEach((device) => {
        const option = document.createElement("option")
        option.value = device.deviceId
        option.text = device.label || `Camera ${select.length}`
        select.appendChild(option)
      })

      select.style.display = "block"
      select.addEventListener("change", (e) => {
        if (e.target.value === "demo") {
          this.startDemoVideo(container)
        } else {
          this.startCamera(container, e.target.value)
        }
      })

      await this.startCamera(container)
    } catch (err) {
      console.error("Error getting camera list:", err)
      await this.startCamera(container)
    }
  }

  async startDemoVideo(container) {
    const data = this.instances.get(container)
    const { video, currentStream } = data

    // Stop any existing stream
    if (currentStream) {
      currentStream.getTracks().forEach((track) => track.stop())
      data.currentStream = null
    }

    // Set up video source
    video.src = "rbc.mp4" // Make sure this file exists in your project
    video.loop = true // Loop the video
    video.play().catch((err) => {
      console.error("Error playing demo video:", err)
      container.innerHTML = "Error: Could not play demo video"
    })
  }

  async startCamera(container, deviceId = null) {
    const data = this.instances.get(container)
    const { video, currentStream } = data

    if (currentStream) {
      currentStream.getTracks().forEach((track) => track.stop())
    }

    try {
      const constraints = {
        video: {
          width: parseInt(container.dataset.width) || 640,
          height: parseInt(container.dataset.height) || 480,
        },
      }

      if (deviceId) {
        constraints.video.deviceId = { exact: deviceId }
      }

      const stream = await navigator.mediaDevices.getUserMedia(constraints)
      data.currentStream = stream
      video.srcObject = stream

      // Wait for video to be ready before playing
      await new Promise((resolve) => {
        video.onloadedmetadata = () => {
          video.width = video.videoWidth
          video.height = video.videoHeight
          resolve()
        }
      })

      await video.play()
    } catch (err) {
      console.error("Error accessing webcam:", err)
      container.innerHTML = "Error: Could not access webcam"
    }
  }

  setupCanvas(container) {
    const data = this.instances.get(container)
    const { video } = data

    const canvas = document.createElement("canvas")
    canvas.className = "scrollWebcamCanvas"
    canvas.width = video.videoWidth || video.width
    canvas.height = video.videoHeight || video.height
    canvas.style.position = "absolute"
    canvas.style.top = "0"
    canvas.style.left = "0"
    canvas.style.zIndex = "1"
    canvas.style.cursor = "crosshair"
    canvas.style.width = "100%" // Make canvas scale with container
    canvas.style.height = "100%"

    container.insertBefore(canvas, container.firstChild.nextSibling)

    data.canvas = canvas
    data.ctx = canvas.getContext("2d")

    this.setupCanvasEventListeners(container)
    this.setupKeyboardShortcuts(container)
  }

  setupHUD(container) {
    const hud = container.querySelector(".scrollWebcamHud")
    if (hud) {
      // Create cell counter element if it doesn't exist
      let cellCounter = hud.querySelector(
        '.scrollWebcamHudElement[data-type="cellCount"]',
      )
      if (!cellCounter) {
        cellCounter = document.createElement("div")
        cellCounter.className = "scrollWebcamHudElement"
        cellCounter.dataset.type = "cellCount"
        cellCounter.style.position = "absolute"
        cellCounter.style.bottom = "10px"
        cellCounter.style.left = "10px"
        cellCounter.style.fontSize = "30px"
        cellCounter.style.textShadow = "1px 1px 2px black"
        hud.appendChild(cellCounter)
      }

      setInterval(() => {
        hud.querySelectorAll(".scrollWebcamHudElement").forEach((el) => {
          if (el.dataset.type === "time") {
            const now = new Date()
            const days = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"]
            const dayOfWeek = days[now.getDay()]
            el.textContent = `${dayOfWeek} ${now.toLocaleDateString()} ${now.toLocaleTimeString()}`
          } else if (el.dataset.type === "cellCount") {
            const data = this.instances.get(container)
            el.textContent = `Cells Counted: ${data.boxes.length}`
          }
        })
      }, 1000)
    }
  }

  setupCanvasEventListeners(container) {
    const data = this.instances.get(container)
    const { canvas } = data

    canvas.addEventListener("mousedown", (e) =>
      this.handleMouseDown(container, e),
    )
    canvas.addEventListener("mousemove", (e) =>
      this.handleMouseMove(container, e),
    )
    canvas.addEventListener("mouseup", (e) => this.handleMouseUp(container, e))
    canvas.addEventListener("dblclick", () => this.clearAllBoxes(container))
  }

  setupKeyboardShortcuts(container) {
    document.addEventListener("keydown", (e) => {
      const data = this.instances.get(container)

      if (e.key === "Escape") {
        data.isDrawing = false
        this.redrawBoxes(container)
      } else if (e.key === "z" && e.ctrlKey) {
        data.boxes.pop()
        this.redrawBoxes(container)
      } else if (e.key === "Delete") {
        this.clearAllBoxes(container)
      } else if (e.key.toLowerCase() === "r") {
        // Toggle automatic box drawing
        if (data.autoDrawInterval) {
          clearInterval(data.autoDrawInterval)
          data.autoDrawInterval = null
          console.log("Stopped auto drawing")
        } else {
          console.log("Started auto drawing")
          let max = 253
          data.autoDrawInterval = setInterval(() => {
            if (max) {
              this.drawRandomBox(container)
              max--
            } else {
              if (Math.random() > 0.92) this.drawRandomBox(container)
            }
          }, 25) // Draw every 500ms
        }
      }
    })
  }

  drawRandomBox(container) {
    const data = this.instances.get(container)
    const { canvas } = data

    // Generate random box dimensions within canvas bounds
    const minSize = 10
    const maxSize = 30
    const width = Math.random() * (maxSize - minSize) + minSize
    const height = Math.random() * (maxSize - minSize) + minSize
    const x = Math.random() * (canvas.width - width)
    const y = Math.random() * (canvas.height - height)

    // Add the box to the collection
    data.boxes.push({ x, y, width, height })

    // Redraw all boxes
    this.redrawBoxes(container)

    // Update cell counter
    const hud = container.querySelector(".scrollWebcamHud")
    const cellCounter = hud.querySelector(
      '.scrollWebcamHudElement[data-type="cellCount"]',
    )
    if (cellCounter) {
      cellCounter.textContent = `Cells Counted: ${data.boxes.length}`
    }
  }

  handleMouseDown(container, e) {
    const data = this.instances.get(container)
    const rect = data.canvas.getBoundingClientRect()

    data.startX = e.clientX - rect.left
    data.startY = e.clientY - rect.top
    data.isDrawing = true
  }

  handleMouseMove(container, e) {
    const data = this.instances.get(container)
    if (!data.isDrawing) return

    const rect = data.canvas.getBoundingClientRect()
    const currentX = e.clientX - rect.left
    const currentY = e.clientY - rect.top

    this.clearCanvas(container)
    this.redrawBoxes(container)
    this.drawBox(
      container,
      data.startX,
      data.startY,
      currentX - data.startX,
      currentY - data.startY,
    )
  }

  handleMouseUp(container, e) {
    const data = this.instances.get(container)
    if (!data.isDrawing) return

    const rect = data.canvas.getBoundingClientRect()
    const endX = e.clientX - rect.left
    const endY = e.clientY - rect.top

    const x = Math.min(data.startX, endX)
    const y = Math.min(data.startY, endY)
    const width = Math.abs(endX - data.startX)
    const height = Math.abs(endY - data.startY)

    if (width > 5 && height > 5) {
      data.boxes.push({ x, y, width, height })
      // Update cell counter immediately after adding a box
      const hud = container.querySelector(".scrollWebcamHud")
      const cellCounter = hud.querySelector(
        '.scrollWebcamHudElement[data-type="cellCount"]',
      )
      if (cellCounter) {
        cellCounter.textContent = `Cells Counted: ${data.boxes.length}`
      }
    }

    data.isDrawing = false
    this.redrawBoxes(container)
  }

  clearCanvas(container) {
    const data = this.instances.get(container)
    const { ctx, canvas } = data
    ctx.clearRect(0, 0, canvas.width, canvas.height)
  }

  redrawBoxes(container) {
    const data = this.instances.get(container)

    this.clearCanvas(container)
    data.boxes.forEach((box) => {
      this.drawBox(container, box.x, box.y, box.width, box.height)
    })
  }

  clearAllBoxes(container) {
    const data = this.instances.get(container)
    data.boxes = []
    this.clearCanvas(container)

    // Update cell counter when clearing boxes
    const hud = container.querySelector(".scrollWebcamHud")
    const cellCounter = hud.querySelector(
      '.scrollWebcamHudElement[data-type="cellCount"]',
    )
    if (cellCounter) {
      cellCounter.textContent = "Cells Counted: 0"
    }
  }
}

// Initialize the app when the DOM is loaded
document.addEventListener("DOMContentLoaded", async () => {
  // Verify TensorFlow.js and COCO-SSD are loaded
  if (typeof tf === "undefined") {
    console.error("TensorFlow.js is not loaded")
    return
  }
  if (typeof cocoSsd === "undefined") {
    console.error("COCO-SSD model is not loaded")
    return
  }

  console.log("TensorFlow.js loaded:", typeof tf !== "undefined")
  console.log("COCO-SSD loaded:", typeof cocoSsd !== "undefined")

  const app = new MitoSiteApp()
  await app.init()
})
